import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

public class rotate2D extends Applet {
 //Arrays for the points and lines to be rotated
 Point2D.Double[] pt = new Point2D.Double[5];
 Line2D.Double[] line = new Line2D.Double[4];

 //Filled circles to draw at the square's center and vertices
 Line2D.Double center = new Line2D.Double(200,300,200,300);
 Line2D.Double[] vertex = new Line2D.Double[4]; //Lines will have length 0
 Color[] vtxColor = new Color[4]; //Different colors for each vertex
 Font f = new Font("Lucida Sans Typewriter Bold", Font.PLAIN, 16);

 //Some Java 2D objects for drawing the square
 Graphics g = null;
 Graphics2D g2 = null;
 BasicStroke lineStroke = new BasicStroke(5,BasicStroke.CAP_ROUND,
   BasicStroke.JOIN_ROUND);
 BasicStroke vtxStroke = new BasicStroke(14,BasicStroke.CAP_ROUND,
   BasicStroke.JOIN_ROUND);
 RenderingHints hints = new RenderingHints(
   RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
 Color lineColor = new Color(43,158,255); //Other lines are blueish
 double angle = 0.0;
 String strAngle = null; //For handling the Java 2.2250738585072012e-308 bug

 //Buttons for Reset and Rotate
 Button btnReset = new Button("Reset");
 Button btnRotate = new Button("Rotate");

 //Label and text field to enter number of degrees
 Label lblDegrees = new Label("Degrees:"); 
 TextField txtDegrees = new TextField(5);

 public void init() {
   //Initialize the square
   for (int i=0; i < 5; i++) {
      pt[i] = new Point2D.Double();
   }
   for (int i=0; i < 4; i++) {
      vertex[i] = new Line2D.Double();
   }
   vtxColor[0] = new Color(240,35,4); //Reddish
   vtxColor[1] = new Color(201,89,220); //Purplish
   vtxColor[2] = new Color(93,236,100); //Greenish
   vtxColor[3] = new Color(236,232,124); //Yellowish
   initializeSquare();

   //Create the lines
   for (int i=0; i < 4; i++) {
      line[i] = new Line2D.Double();
   }

   //Create Reset action: Re-initialize the square and draw
   btnReset.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         g = getGraphics();
         initializeSquare();
         drawSquare(g,pt);
      }
   });

   //Create Rotate action
   btnRotate.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         rotateSquare();
      }
   });

   //Make the Enter key in the degrees textfield act like the Rotate button
   txtDegrees.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent ke) {
         if(ke.getKeyCode() == KeyEvent.VK_ENTER) {
            rotateSquare();
         }
      }
   });

   this.add(btnReset); this.add(btnRotate);
   this.add(lblDegrees); this.add(txtDegrees);
 }

 public void paint(Graphics g) {
   drawSquare(g,pt);
 }

 //Method for initializing the square
 public void initializeSquare() {
   pt[0].setLocation(100,200);
   pt[1].setLocation(300,200);
   pt[2].setLocation(300,400);
   pt[3].setLocation(100,400);
   pt[4].setLocation(100,200);
   for (int i=0; i < 4; i++) {
      vertex[i].setLine(pt[i],pt[i]); //Line has length 0 to simulate a point
   }
}

 //Method for rotating the square
 public void rotateSquare() {
   g = getGraphics();
   //Handle the 2.2250738585072012e-308 -> double bug: for now don't allow
   //any scientific notation input until the bug is fixed
   strAngle = txtDegrees.getText().trim().toLowerCase(); //Angle input from user
   if (strAngle.indexOf("e") != -1) { //Input is not OK
      strAngle = "0.0";
   }
   angle = deg2rad(Double.parseDouble(strAngle)); //Use radians

   //Note that in the AWT coordinate system, positive y points downward, not
   //upward. So we need the following sequence to rotate each point around the
   //square's center (200,300) as if it were (0,0) in Cartesian coordinates:
   //(x,y) -> (x,-y) -> (x-200,-y+300) -> rotate to (x',y') -> (x',-y')
   //-> (x'+200,-y'+300)
   for (int i=0; i < 5; i++) {
      //First reflect around y-axis then shift left by 200 and down by 300
      pt[i].setLocation(pt[i].x - 200,-pt[i].y + 300);
      //Now rotate (this is the trigonometry part:)
      pt[i].setLocation(Math.cos(angle)*pt[i].x - Math.sin(angle)*pt[i].y,
                       Math.sin(angle)*pt[i].x + Math.cos(angle)*pt[i].y);
      //Then untranslate to old coordinate system
      pt[i].setLocation(pt[i].x + 200,-pt[i].y + 300);
   }
   drawSquare(g,pt);
 }

 //Method for drawing the new set of points
 public void drawSquare(Graphics g, Point2D.Double[] pt) {
   g2 = (Graphics2D)g;
   hints.put(RenderingHints.KEY_RENDERING,RenderingHints.VALUE_RENDER_QUALITY);
   g2.setRenderingHints(hints); //This makes the square look better
   g2.setColor(Color.black); //Set background color
   g2.fillRect(0,50,getSize().width,getSize().height - 50);

   //Draw the square
   g2.setStroke(lineStroke);
   g2.setColor(Color.orange); //Draw the top line
   line[0].setLine(pt[0],pt[1]);
   g2.draw(line[0]);
   g2.setColor(lineColor); //Draw the other lines
   for (int i=1; i < 4; i++) {
      line[i].setLine(pt[i],pt[i+1]);
      g2.draw(line[i]);
   }
   g2.draw(center);
   g2.setStroke(vtxStroke);
   g2.setFont(f);
   for (int i=0; i < 4; i++) { //Draw the vertex points
      g2.setColor(vtxColor[i]);
      vertex[i].setLine(pt[i],pt[i]);
      g2.draw(vertex[i]);
      g2.drawString("\u2219  P" + (i+1) + " = (" + String.format("%.3f",pt[i].x
       - 200) + "," + String.format("%.3f",300 - pt[i].y) + ")",60,(510+i*20));
   }
 }

 //Method for converting degrees to radians
 public double deg2rad(double angle) {
   return angle * Math.PI / 180;
 }
}
